package Atlas::Project::Trace;
$VERSION=1.0;

use strict;
use Atlas::Utility::ObjectAttribute;

use vars qw(%tracename_chem_to_phrap %tracename_phrap_to_chem %chem_to_direction);

%tracename_chem_to_phrap=(
			  'U'=>'s',
			  'R'=>'r',
			  'T'=>'x',
			  'Y'=>'y',
			  );

%tracename_phrap_to_chem=reverse(%tracename_chem_to_phrap);

%chem_to_direction=('D'=>'F',
		    'E'=>'R',
		    'U'=>'F',
		    'R'=>'R',
		    'Q'=>'R',
		    );

my @_trace_attribute=(
	     'name',
	     # Where the file are
	     'abi_path', 
	     'scf_path',
	     'exp_path', 
	     #potentially other attributs including quality, sequence
	     'original_str',  #the very original string passed in for a trace
	     'original_name',
	     'dir',
             'ext',
	     'compressed',
             'coslib',
	     'sample',
             'chem',
	     'attempt',
	     'contig_strand',
	     'contig_start',
	     'contig_end',
	     'contig',
	     'format',
	     'project',
    	     'center',
	     'direction',
	     );

&Atlas::Utility::ObjectAttribute::declare_rw(@_trace_attribute);


#------------------------------------
#constructor
#it receive either empty string or a tracename
#if empty str
#    a empty hash and blessed
#else
#    parse the string, store right info
#______________________________________________
sub create {
    my($class)=shift;
    my($tracename)=shift;

    my($trace)=+{
	#empty initially
    };
    bless($trace, $class);

    if(defined $tracename) {
	$trace->_parsename($tracename);
    }
    return $trace;
}

#-------------------------------------
#give the length of a trace, always positive
#-------------------------------------
sub length {
    my($trace)=shift;
    
    my $length=abs($trace->contig_end - $trace->contig_start)+1;
    return $length;
}

#------------------------------------------
#give the template name of a trace
#-----------------------------------------
sub mapgap_group {
    my($trace)=shift;

    return if(! $trace->coslib() || ! $trace->sample() );
    my $mapgap_group=($trace->coslib).'_'.($trace->sample);
    return $mapgap_group;
}

#------------------------------------------
#parse the input string into appropriate info
#the string can be a file name or a tracename
#-------------------------------------------
sub _parsename {

    my($trace)=shift;
    my($tracename)=shift;

    my($dir,$name,$ext,$compressed); #
    my($attempt,$chem,$contig_strand,$format,$sample,$coslib); #

    #
    goto SKIP if ($tracename=~/^gnl\|ti/);
    $format='';

    # FIXME: assume the positive strand
    $contig_strand='';

    ($dir,$name,$ext,$compressed)=$trace->_filename_to_tracename($tracename);


    

    # A11U0001 to R91U9999 to SA1U9999 to ZZZ1U99999
    if ($name=~m/^
	([a-r][o0-9]|[s-z][a-z]|[a-z0-9]{4}[a-z])
	([o0-9]{1,2})
	([a-z])
	([o0-9]+)
	(.comp)?$
	/oix) {
	$coslib=$1;
	$attempt=$2;
	$chem=$3;
	$sample=$4;
	$contig_strand=$5;
	$format='bcm_1';
    }
    #AAAAP1D67899.scf.comp
    elsif( $name=~m/^([a-z]{5})([0-9]{1,2})([a-z])([0-9]+)/oxi ) {
	$coslib=$1;
	$attempt=$2;
	$chem=$3;
	$sample=$4;
	$contig_strand='comp' if( $name =~ /\.comp/i);
	$format='bcm_1';
    }

    # SA9999U1
    elsif ($name=~m/^([s-z][a-z]+)([o0-9]+)
	   ([a-z])([o0-9])(.comp)?$/iox) {
	$coslib=$1;
	$sample=$2;
	$chem=$3;
	$attempt=$4;
	$contig_strand=$5;
	$format='bcm_2';
    }

    # Phrap name  SS_0000.s1
    elsif ($name=~/^([a-r][o0-9]|[t-z][a-z]|[a-z0-9]{4}[a-z])_
	   ([o0-9]+).(\w)([o0-9]{1,2})(.comp)?$/oix) {
	$coslib=$1;
	$sample=$2;
	$chem=$tracename_phrap_to_chem{$3}||$3;
	$attempt=$4;
	$contig_strand=$5;
	$format='bcm_phrap_1';
    }

    # Phrap name  SS1_0000.s, SSSS1_1234567.D.comp
    elsif ($name=~/^([a-r][o0-9]|[t-z][a-z]|[a-z0-9]{4}[a-z])([o0-9]{1,2})_
	   ([o0-9]+).(\w)(.comp)?$/oix) {
	$coslib=$1;
	$attempt=$2;
	$sample=$3;
	$chem=$tracename_phrap_to_chem{$4}||$4;
	$contig_strand=$5;
	$format='bcm_phrap_2';
    }


    # phrap rat reads SSSS1_1234567A.D.comp
    elsif ($name=~/^([a-r][o0-9]|[t-z][a-z]|[a-z0-9]{4}[a-z])([o0-9]{1,2})_
	   ([o0-9]+)[a-z].(\w)(.comp)?$/oix) {
	$coslib=$1;
	$attempt=$2;
	$sample=$3;
	$chem=$tracename_phrap_to_chem{$4}||$4;
	$contig_strand=$5;
	$format='bcm_rat_384';
    }


    #JGI reads for bacteria like XBM3456.x1
    elsif($name=~/([a-z]{3})(\d+)\.([a-z])(\d)/io) {
	$coslib=$1;
	$sample=$2;
	$chem=$3;
	$attempt=$4;
	$format='jgi'
    }

    #drosophila ones like 12345F234_45678
    elsif($name=~/^(\d+)([F|R])(\d+)/io) {
	$coslib=$1;
	$sample=$3;
	$chem=$2;
	$format='celera_dr';
    }

    #NHGRI name for rat project auv22e04 or rj22e04
    elsif($name =~ /^([a-z]{2,3})(\d+[a-z]\d+)/io) {
	$coslib=$1;
	$sample=$2;
	$format='nhgri_rat';
    }

    # Fix up parts of the names if we parsed.
    if ($format) {
	# Fix it up
	$coslib=uc($coslib);
	$chem=uc($chem) if (defined $chem);

	if (defined $attempt) {
	    $attempt=~tr/oO/00/;
	    $attempt=sprintf("%d",$attempt);
	}

	if(defined $sample) {
	    $sample=~tr/oO/00/;
	    $sample=sprintf("%04d",$sample) if($sample=~/^\d+$/);
	}
	
	if ($contig_strand) {
	    $contig_strand='-';
	}
	else {
	    $contig_strand='+';
	}
    }

  SKIP:
    $trace->{'original_str'}=$tracename;
    $trace->{'original_name'}=$name;
    $trace->{'dir'}=$dir;
    $trace->{'ext'}=$ext;
    $trace->{'compressed'}=$compressed;
    $trace->{'coslib'}=$coslib;
    $trace->{'attempt'}=$attempt;
    $trace->{'chem'}=$chem;
    $trace->{'sample'}=$sample;
    $trace->{'contig_strand'}=$contig_strand;
    $trace->{'format'}=$format;
    $trace->{'direction'}=$chem_to_direction{$chem} if (defined $chem && exists $chem_to_direction{$chem});
}

#-------------------------------------------
#check the input string type(file?, file type?)
#return info about this string
#--------------------------------------------
sub _filename_to_tracename {
    my($self)=shift;
    my($name)=shift;

    # Clean up prefixes and suffixes
    if ($name=~/^
	(.*\/)?    # dir
	([^\/]+?) # name
	(.abi|.scf|.acf|.ace|.exp|.gb|.xdap)? # extention
	(.gz|.Z)? # compressed?
	$/iox) {
	# $dir=$1; $name=$2; $ext=$3; $compressed=$4;
	($1,$2,$3,$4);
    }
    else {
	('',$name,'','');
    }
}

1;

__END__

=head1 NAME
Atlas::Project::Trace - a trace class contain infomation of a read

=head1 VERSION
1.0

=MODIFICATION
APRIL 15, 2001

=head1 SYNOPSIS

use Atlas::Project::Trace;

Atlas::Project::Trace->create();

Atlas::Project::Trace->create($tracename);

$trace->length();

$trace->mapgap_group();

$trace->attempt();

$trace->chem();

$trace->contig_strand();

$trace->coslib();

$trace->dir();

$trace->ext();

$trace->format();

$trace->original_name();

$trace->original_str();

$trace->sample();

=head1 DESCRIPTION

=head2 Overview

=over 4

=item * 

Creation of the object:

Either an empty one or one start with a name

=item *

Accessor and Mutator

all attributes for the trace object are readable and writable.

Their value will be changed if you pass a value into the function

=back

=head2 Constructor and initialization

Atlas::Project::Trace->create();

Atlas::Project::Trace->create($tracename);

=head2 Functions

$trace->length();  <length of the read>

$trace->mapgap_group();  <library name concate with sample number, essentially template name>

=head2 Accessor and Mutators

$trace->attempt();  <which round this sequence have been tried>

$trace->chem();  <chemistry used>

$trace->contig_strand();  <orientation relative to the contig it is in>

$trace->coslib();  <library name>

$trace->dir();  <trace file directory path>

$trace->ext();  <file extention, such as .abi, .exp, etc>

$trace->format(); <file format, such as phrap, bcm_1, etc>

$trace->original_name(); <sequence original name>

$trace->original_str();  <the initial string passed in when the object created>

$trace->sample();  <the clone number>

=head1 ENVIRONMENT

UNIX

=head1 FILES

Other Modules used:

Atlas::Utility::ObjectAttribute

=head1 LOCATION



=head1 AUTHOR

rui chen

ruichen@bcm.tmc.edu

==cut



